#!/usr/bin/env python

from __future__ import division

def main(summary_file):
    relations = []
    for line in open(summary_file):
        if line == '\n':
            relations.append(Relation(name, running_time, test_instances,
                missing, correct_at_1, correct_at_3, correct_at_5))
            continue
        key, value = line.strip().split(': ', 1)
        if key == 'Relation':
            name = value
        elif key == 'Running time':
            running_time = float(value)
        elif key == 'Total test instances':
            test_instances = int(value)
        elif key == 'Missing predictions':
            missing = int(value)
        elif key == 'Correct at 1':
            correct_at_1 = float(value.split()[1][1:-1])
        elif key == 'Correct at 3':
            correct_at_3 = float(value.split()[1][1:-1])
        elif key == 'Correct at 5':
            correct_at_5 = float(value.split()[1][1:-1])
    relations.sort(key=lambda x: (x.correct_at_1, -x.test_instances),
                   reverse=True)
    print_best_performing_relations(relations, 'summary.txt')
    print_averages(relations)


def print_best_performing_relations(relations, outfile=None):
    result = '%-70s\t%10s\t%10s\t%10s\n' % ('Relation name',
                                            'precision@1',
                                            '#instances',
                                            '#missing')
    for relation in relations:
        result += '%-70s\t%10.3f\t%10d\t%10d\n' % (relation.name,
                                                   relation.correct_at_1,
                                                   relation.test_instances,
                                                   relation.missing)
    if outfile:
        open(outfile, 'w').write(result)
    else:
        print result


def print_averages(relations):
    running_times = [x.running_time for x in relations]
    test_instances = [x.test_instances for x in relations]
    missing = [x.missing for x in relations]
    correct_at_1 = [x.correct_at_1 for x in relations]
    correct_at_3 = [x.correct_at_3 for x in relations]
    correct_at_5 = [x.correct_at_5 for x in relations]
    print 'Average of', len(relations), 'relations'
    print 'Running time:', sum(running_times)/len(running_times)
    print 'Max running time:', max(running_times)
    print 'Test instances:', sum(test_instances)/len(test_instances)
    print 'Max test instances:', max(test_instances)
    print 'Missing:', sum(missing)/len(missing)
    print 'Correct at 1:', sum(correct_at_1)/len(correct_at_1)
    print 'Correct at 3:', sum(correct_at_3)/len(correct_at_3)
    print 'Correct at 5:', sum(correct_at_5)/len(correct_at_5)
    weights = [x.test_instances - x.missing for x in relations]
    total_weight = sum(weights)
    weighted_1 = sum(x[0]*x[1] for x in zip(weights, correct_at_1))
    weighted_3 = sum(x[0]*x[1] for x in zip(weights, correct_at_3))
    weighted_5 = sum(x[0]*x[1] for x in zip(weights, correct_at_5))
    print 'Weighted correct at 1:', weighted_1 / total_weight
    print 'Weighted correct at 3:', weighted_3 / total_weight
    print 'Weighted correct at 5:', weighted_5 / total_weight


class Relation(object):
    def __init__(self, name, running_time, test_instances, missing,
            correct_at_1, correct_at_3, correct_at_5):
        self.name = name
        self.running_time = running_time
        self.test_instances = test_instances
        self.missing = missing
        self.correct_at_1 = correct_at_1
        self.correct_at_3 = correct_at_3
        self.correct_at_5 = correct_at_5


if __name__ == '__main__':
    from optparse import OptionParser
    parser = OptionParser()
    parser.add_option('-f', '--filename',
            dest='summary_file',
            help='Path to summary file',
            )
    opts, args = parser.parse_args()
    main(opts.summary_file)

# vim: et sw=4 sts=4
