#!/usr/bin/env python

from collections import defaultdict
from counter import Index

def main(embeddings_file, graph_file, out_dir, concat):
    graph_base = '/'.join(graph_file.split('/')[:-1]) + '/'
    node_index = Index.read_from_file(open(graph_base + 'node_dict.tsv'))
    embeddings = {}
    print 'Reading embeddings file'
    for line in open(embeddings_file):
        fields = line.strip().split('\t')
        verb = fields[0]
        vector = [float(x) for x in fields[1:]]
        embeddings[fields[0]] = embeddings_to_edges(vector, concat)
    edges = set()
    print 'Reading graph file and converting edges'
    i = 0
    for line in open(graph_file):
        i += 1
        if i % 1000000 == 0:
            print i
        source, target, relation = line.strip().split('\t')
        if not relation.startswith('"'):
            edges.add((source, target, relation))
            continue
        relation = relation[1:-1]
        for embedding in embeddings[relation]:
            edges.add((source, target, embedding))
    print 'Outputting new files'
    edge_index = Index()
    edge_file = open(out_dir + 'labeled_edges.tsv', 'w')
    edge_id_file = open(out_dir + 'graph_chi/edges.tsv', 'w')
    for source, target, relation in edges:
        source_id = node_index.getIndex(source)
        target_id = node_index.getIndex(target)
        edge_id = edge_index.getIndex(relation)
        edge_file.write('%s\t%s\t%s\n' % (source, target, relation))
        edge_id_file.write('%d\t%d\t%d\n' % (source_id, target_id, edge_id))
    edge_file.close()
    edge_id_file.close()
    edge_index.save_to_file(open(out_dir + 'edge_dict.tsv', 'w'))
    node_index.save_to_file(open(out_dir + 'node_dict.tsv', 'w'))



def embeddings_to_edges(vector, concat):
    if concat:
        return [concat_highest(vector)]
    else:
        return list_highest(vector)


def concat_highest(vector):
    with_abs = [(abs(x), x, i+1) for i, x in enumerate(vector)]
    with_abs.sort(reverse=True)
    to_keep = 2
    value = ''
    for abs_value, orig, index in with_abs[:to_keep]:
        if abs_value == orig:
            value += '+L%d:' % index
        else:
            value += '-L%d:' % index
    return value[:-1]


def list_highest(vector):
    with_abs = [(abs(x), x, i+1) for i, x in enumerate(vector)]
    with_abs.sort(reverse=True)
    to_keep = 2
    edges = []
    for abs_value, orig, index in with_abs[:to_keep]:
        if abs_value == orig:
            edges.append('+L%d' % index)
        else:
            edges.append('-L%d' % index)
    return edges


if __name__ == '__main__':
    from optparse import OptionParser
    parser = OptionParser()
    parser.add_option('-e', '--embeddings',
            dest='embeddings',
            help='Path to embeddings file',
            )
    parser.add_option('-g', '--graph-file',
            dest='graph_file',
            help='Path to graph file',
            )
    parser.add_option('-o', '--out-dir',
            dest='out_dir',
            help='Path to out directory',
            )
    parser.add_option('-c', '--concat',
            dest='concat',
            action='store_true',
            help='Concatenate top embeddings (instead of listing them)'
            )
    opts, args = parser.parse_args()
    main(opts.embeddings, opts.graph_file, opts.out_dir, opts.concat)

# vim: et sw=4 sts=4
