#!/usr/bin/env python

from __future__ import division

def main():
    relations = [
            'concept:animalistypeofanimal',
            'concept:athleteplaysforteam',
            'concept:athleteplaysinleague',
            'concept:cityliesonriver',
            'concept:citylocatedincountry',
            'concept:companyceo',
            'concept:countryhascompanyoffice',
            'concept:drughassideeffect',
            'concept:headquarteredin',
            'concept:locationlocatedwithinlocation',
            'concept:publicationjournalist',
            'concept:roomcancontainfurniture',
            'concept:stadiumlocatedincity',
            'concept:teamplaysagainstteam',
            'concept:writerwrotebook',
            ]
    directories = [
            'fixed_split/nell/results/',
            'fixed_split/svo_nell/results/',
            'fixed_split/pca_c_svo_nell/results/',
            'fixed_split/pca_d_svo_nell/results/',
            ]
    cutoffs = [
            #(2.94444, .95),
            #(2.19722, .90),
            #(1.0986125, .75),
            (0.4054651, .6),
            ]
    print '\\begin{tabular}{|l|r|r|r|r|}'
    print '\\hline'
    for d in directories:
        method = d.split('/')[1]
        print '&%s' % method,
    print '\\\\\n\\hline'
    for c, percent in cutoffs:
        averages = []
        for d in directories:
            averages.append([])
        for r in relations:
            print '%s' % r.split(':')[-1],
            for i, d in enumerate(directories):
                scores_file = d + r + '/scores.tsv'
                preds = []
                for line in open(scores_file):
                    fields = line.split('\t')
                    if len(fields) != 4 or not fields[1]:
                        continue
                    score = float(fields[2])
                    correct = '*' in fields[3]
                    preds.append((score, correct))
                total = len([x for x in preds if x[1]])
                predicted_at_cutoff = [x for x in preds if x[0] > c]
                if not predicted_at_cutoff:
                    print '& 0',
                    continue
                correct = len([x for x in predicted_at_cutoff if x[1]])
                precision = correct / len(predicted_at_cutoff)
                recall = correct / total
                f1 = 2 * precision * recall / (precision + recall)
                print '& %.2f' % (f1),
                averages[i].append((precision, recall))
            print '\\\\\n\\hline'
        print 'Average',
        for i, d in enumerate(directories):
            precs = [x[0] for x in averages[i]]
            recs = [x[1] for x in averages[i]]
            prec = sum(precs) / len(precs)
            rec = sum(recs) / len(relations)
            f1 = prec * rec * 2 / (prec + rec)
            print '& %.3f & %.3f & %.3f' % (sum(precs)/len(precs),
                    sum(recs)/len(relations), f1),
        print '\\\\\n\\hline'
    print '\\end{tabular}'


if __name__ == '__main__':
    main()

# vim: et sw=4 sts=4
