#!/usr/bin/env python

from __future__ import division
import os
from evilplot import Plot, Points

def main(outfile, directory, aggregate, compare, show):
    p = Plot()
    if compare:
        if not directory:
            directory = ''
        directories = [
                'fixed_split/nell/results/' + directory,
                'fixed_split/svo_nell/results/' + directory,
                'fixed_split/pca_c_svo_nell/results/' + directory,
                'fixed_split/pca_d_svo_nell/results/' + directory,
                ]
        keys = {}
        keys[directories[0]] = 'n:'
        keys[directories[1]] = 'sn:'
        keys[directories[2]] = 'pcsn:'
        keys[directories[3]] = 'pdsn:'
    else:
        directories = [directory]
    for d in directories:
        predictions = []
        for root, dirs, files in os.walk(d):
            for f in files:
                if f == 'scores.tsv':
                    filename = root + '/' + f
                    if aggregate:
                        preds = get_predictions_from_file(filename)
                        predictions.extend(preds)
                    else:
                        points_name = filename.split('/')[-2].split(':')[-1]
                        if compare:
                            points_name = keys[d] + points_name
                        p.append(get_points_for_file(filename, points_name))
        if aggregate:
            dir_name = d.split('/')[1] + directory
            points = get_points_from_predictions(predictions, dir_name)
            if points:
                p.append(points)
            else:
                print 'WARNING: no points for', dir_name
    p.write(outfile)
    if show:
        p.show()


def get_points_for_file(filename, points_name):
    preds = get_predictions_from_file(filename)
    return get_points_from_predictions(preds, points_name)


def get_predictions_from_file(filename):
    predictions = []
    for line in open(filename):
        fields = line.split('\t')
        if len(fields) != 4 or not fields[1]:
            continue
        score = float(fields[2])
        correct = '*' in fields[3]
        predictions.append((score, correct))
    return predictions


def get_points_from_predictions(predictions, rel_name):
    predictions.sort(reverse=True)
    total_possible = sum(1 for x in predictions if x[1])
    total_seen = 0
    prec_rec = []
    total_correct = 0
    for score, correct in predictions:
        total_seen += 1
        if correct:
            total_correct += 1
        prec = total_correct / total_seen
        rec = total_correct / total_possible
        prec_rec.append((rec, prec))
    if not prec_rec:
        return None
    return Points(prec_rec, title=rel_name, style='lines')


if __name__ == '__main__':
    from optparse import OptionParser
    parser = OptionParser()
    parser.add_option('-d', '--dir',
            dest='directory',
            default='',
            help='Path to directory to search for files',
            )
    parser.add_option('-a', '--aggregate',
            dest='aggregate',
            default=False,
            action='store_true',
            help='Combine predictions for all relations (only with -d)',
            )
    parser.add_option('-c', '--compare',
            dest='compare',
            default=False,
            action='store_true',
            help='Compare predictions across runs',
            )
    parser.add_option('-o', '--outfile',
            dest='outfile',
            default='plot.gpi',
            help='File to save the resultant plot in',
            )
    parser.add_option('-s', '--show',
            dest='show',
            default=False,
            action='store_true',
            help='Show results, in addition to saving them to disk',
            )
    opts, args = parser.parse_args()
    if opts.directory:
        outfile = opts.directory.split(':')[-1] + '.gpi'
    else:
        outfile = opts.outfile
    main(outfile, opts.directory, opts.aggregate, opts.compare, opts.show)

# vim: et sw=4 sts=4
