#!/usr/bin/env python

from __future__ import division

from counter import Index
from datetime import datetime
from dirutil import create_dirs_and_open
from subprocess import Popen
import os
import random

def main(code_dir, base_dir, shards, relation_files=[]):
    graph_file = base_dir + 'graph_chi/edges.tsv'
    #graph_file = '/dev/shm/graph_chi/edges.tsv'
    if not relation_files:
        for root, dirs, files in os.walk(base_dir + 'relations'):
            for f in files:
                relation_name = f.split('_')[0]
                if 'latitudelong' in relation_name: continue
                if 'atdate' in relation_name: continue
                if 'haswikipediaurl' in relation_name: continue
                # Something was up with these relations, where all of the path
                # types had 0 weight.  Not sure what happened, but it should
                # probably be looked into.
                if 'proxyfor' in relation_name: continue
                if 'proxyof' in relation_name: continue
                if 'subpartof' in relation_name: continue
                if 'locationcontainslocation' in relation_name: continue
                if 'createdbyagent' in relation_name: continue
                if 'subpartoforganization' in relation_name: continue
                #if os.path.exists(base_dir + 'results/' + relation_name +
                        #'/weights.tsv'):
                    #continue
                relation_files.append(root + '/' + f)
            #relation_files.extend(root + '/' + x for x in files)
        random.shuffle(relation_files)
    relation_ranges = {}
    for line in open(base_dir + 'ranges.tsv'):
        relation, rel_range = line.strip().split('\t')
        relation_ranges[relation] = rel_range
    inverses_file = base_dir + 'inverses_ids.tsv'
    walkspersource = 500
    path_finder_iters = 4
    paths_to_keep = 750
    walks_per_path = 100
    accept_policy = 'all-targets'
    path_policy = 'paired-only'
    use_unseen = 'true'
    results_file = create_dirs_and_open(base_dir + 'results/summary.txt')
    for relation_file in relation_files:
        print relation_file
        relation = relation_file.split('/')[-1].split('_')[0]
        rel_range = relation_ranges[relation]
        range_file = base_dir + 'category_instances/' + rel_range
        to_ignore = base_dir + 'to_ignore/' + relation
        results_dir = base_dir + 'results/' + relation + '/'
        base_split_dir = os.getcwd() + '/splits/' + relation + '/'
        training_file = base_split_dir + 'training_examples.tsv'
        testing_file = base_split_dir + 'testing_examples.tsv'
        command = ['java', '-Xmx20g', '-cp', 'build/classes:lib/*',
                'edu.cmu.ml.rtw.pra.PraDriver',
                '-g', graph_file,
                '-n', str(shards),
                '-t', range_file,
                '--toignore', to_ignore,
                '--inverses', inverses_file,
                '--acceptpolicy', accept_policy,
                '--pathpolicy', path_policy,
                '-o', results_dir,
                '-d', relation_file,
                # Uncomment these lines (and comment out the -d line above) for
                # running with a fixed training/testing split.
                #'--training', training_file,
                #'--testing', testing_file,
                '--walkspersource', str(walkspersource),
                '--niters', str(path_finder_iters),
                '--numpaths', str(paths_to_keep),
                '--walksperpath', str(walks_per_path),
                '--useunseenasnegative', use_unseen,
                ]
        print 'Running PRA for relation:', relation
        results_file.write('Relation: %s\n' % relation)
        results_file.flush()
        start = datetime.now()
        proc = Popen(command, cwd=code_dir)
        returncode = proc.wait()
        if returncode != 0:
            print 'Process ended with a nonzero return code!  Quiting...'
            results_file.close()
            return
        end = datetime.now()
        delta = end - start
        seconds = delta.seconds + delta.microseconds / 1000000.0
        print 'PRA finished in', seconds, 'seconds'
        results_file.write('Running time: %.2f\n' % seconds)
        try:
            translate_and_score_results(results_dir, base_dir, relation,
                    results_file)
        except:
            results_file.write('Error!')
            print 'Error translating and scoring results for', relation
            import traceback
            print traceback.format_exc()
        results_file.write('\n')
    results_file.close()


def translate_and_score_results(results_dir, base_dir, relation, results_file):
    scores_file = results_dir + 'scores.tsv'
    node_dict = Index.read_from_file(open(base_dir+'node_dict.tsv'))
    relation_dir = base_dir + 'results/' + relation + '/'
    translated_scores_file = relation_dir + 'scores.tsv.translated'
    out_scores = create_dirs_and_open(translated_scores_file)
    missing_predictions = 0
    total = 0
    correct_at_1 = 0
    correct_at_3 = 0
    correct_at_5 = 0
    correct_index = 0
    num_predictions = []
    for line in open(scores_file):
        if line == '\n':
            total += 1
            out_scores.write(line)
            num_predictions.append(correct_index)
            correct_index = 0
            continue
        correct_index += 1
        source, target, score, correct = line.split('\t')
        source = node_dict.getString(int(source))
        if not target:
            out_scores.write(source + '\t\t\t\n')
            correct_index = 0
            missing_predictions += 1
            continue
        target = node_dict.getString(int(target))
        out_scores.write(source+'\t'+target+'\t'+score+'\t'+correct)
        if '*' in correct:
            if correct_index == 1:
                correct_at_1 += 1
            if correct_index <= 3:
                correct_at_3 += 1
            if correct_index <=5:
                correct_at_5 += 1
    out_scores.close()
    #proc = Popen('mv %s/*.tsv %s' % (code_dir, relation_dir), shell=True)
    #proc.wait()
    translate_files(base_dir, relation_dir)
    predicted = total - missing_predictions
    percent_at_1 = correct_at_1 / predicted if predicted else 0
    percent_at_3 = correct_at_3 / predicted if predicted else 0
    percent_at_5 = correct_at_5 / predicted if predicted else 0
    average_predictions = sum(num_predictions)/len(num_predictions)
    print 'Statistics:'
    print 'Total test instances:', total
    print 'Average number of predictions:', average_predictions
    print 'Missing predictions:', missing_predictions
    print 'Correct at 1: %d (%.3f)' % (correct_at_1, percent_at_1)
    print 'Correct at 3: %d (%.3f)' % (correct_at_3, percent_at_3)
    print 'Correct at 5: %d (%.3f)' % (correct_at_5, percent_at_5)
    results_file.write('Total test instances: %d\n' % total)
    results_file.write('Average number of predictions: %.2f\n' %
            average_predictions)
    results_file.write('Missing predictions: %d\n' % missing_predictions)
    results_file.write('Correct at 1: %d (%.3f)\n'%(correct_at_1,percent_at_1))
    results_file.write('Correct at 3: %d (%.3f)\n'%(correct_at_3,percent_at_3))
    results_file.write('Correct at 5: %d (%.3f)\n'%(correct_at_5,percent_at_5))


def translate_files(base_dir, relation_dir):
    import translate_paths
    translate_paths.main(relation_dir + 'paths.tsv',
            base_dir + 'edge_dict.tsv')
    import translate_weights
    translate_weights.main(relation_dir + 'weights.tsv',
            base_dir + 'edge_dict.tsv')
    import translate_matrix
    translate_matrix.main(relation_dir + 'matrix.tsv',
            relation_dir + 'paths.tsv.translated',
            base_dir + 'node_dict.tsv')
    translate_matrix.main(relation_dir + 'positive_matrix.tsv',
            relation_dir + 'paths.tsv.translated',
            base_dir + 'node_dict.tsv')
    translate_matrix.main(relation_dir + 'negative_matrix.tsv',
            relation_dir + 'paths.tsv.translated',
            base_dir + 'node_dict.tsv')
    translate_matrix.main(relation_dir + 'test_matrix.tsv',
            relation_dir + 'paths.tsv.translated',
            base_dir + 'node_dict.tsv')


if __name__ == '__main__':
    from optparse import OptionParser
    parser = OptionParser()
    parser.add_option('-b', '--base',
            dest='base',
            help='Base directory to get data files from',
            )
    parser.add_option('-s', '--shards',
            dest='shards',
            help='Number of shards to use',
            )
    opts, args = parser.parse_args()
    base = opts.base
    if not base:
        parser.print_help()
        exit(-1)
    num_shards = opts.shards
    relations_to_run = [
            'concept:citycapitalofcountry',
            #'concept:teamwontrophy',
            #'concept:vehicleistypeofvehicle',
            #'concept:politicianholdsoffice',
            #'concept:athleteplayssport',
            #'concept:economicsectorcompany',
            ]
    #relations_to_run = [base + 'relations/' + x + '_sources.tsv'
            #for x in relations_to_run]
    relations_to_run = [base + 'training_examples/' + x
            for x in relations_to_run]
    #for line in open('relations_to_run.txt'):
        #relations_to_run.append(base+'relations/'+line.strip()+'_sources.tsv')
    main('/home/mg1/clone/OntologyLearner/', base, num_shards, relations_to_run)
    #main('/home/mg1/clone/OntologyLearner/', base, num_shards, [
            #base+'relations/concept:moviestaractor_sources.tsv',
            #base+'relations/concept:citycapitalofstate_sources.tsv',
            #base+'relations/concept:writerwrotebook_sources.tsv',
            #])

# vim: et sw=4 sts=4
